<?php

namespace NoccyLabs\TreeDumper;

class TreeDumper
{
    const STYLE_ASCII = "ascii";
    const STYLE_UTF8 = "utf8";

    const PREFIX_BRANCH = 0;
    const PREFIX_BRANCH_LAST = 1;
    const PREFIX_STEM = 2;
    const PREFIX_STEM_LAST = 3;

    protected $chars = [];

    public function __construct($style=self::STYLE_ASCII)
    {
        $this->setStyle($style);
    }

    public function setStyle($style)
    {
        switch ($style) {
            case self::STYLE_ASCII:
                $this->chars = [
                    self::PREFIX_BRANCH         => " +--",
                    self::PREFIX_BRANCH_LAST    => " +--",
                    self::PREFIX_STEM           => " |  ",
                    self::PREFIX_STEM_LAST      => "    ",
                ];
                break;
            case self::STYLE_UTF8:
                $this->chars = [
                    self::PREFIX_BRANCH         => $this->uchars(" \u251c\u2500\u2500"),
                    self::PREFIX_BRANCH_LAST    => $this->uchars(" \u2514\u2500\u2500"),
                    self::PREFIX_STEM           => $this->uchars(" \u2502  "),
                    self::PREFIX_STEM_LAST      => $this->uchars("    "),
                ];
                break;
        }
    }

    public function dump($array_or_iter, $return=false)
    {
        if ($array_or_iter instanceof \Iterator) {
            $data = iterator_to_array($array_or_iter);
        } elseif (is_array($array_or_iter)) {
            $data = $array_or_iter;
        } else {
            return;
        }

        $tree = [];
        foreach ($data as $root=>$branch) {
            $tree[] = $root;
            $subtree = $this->renderBranchRecursive($branch);
            $tree = array_merge($tree, $subtree);
        }
        if ($return) {
            return join("\n", $tree) . "\n";
        }
        echo join("\n", $tree) . "\n";

    }

    protected function renderBranchRecursive(array $branch)
    {
        $output = [];

        foreach ($branch as $key=>$value) {
            $is_last = ($value == end($branch));

            if (is_array($value)) {
                // render branch, key is label
                $label = $key;
            } else {
                // render branch, value is label (unless null)
                $label = ($value === NULL) ? $key : $value;
            }

            $output[] = $this->chars[$is_last?self::PREFIX_BRANCH_LAST:self::PREFIX_BRANCH] . $label;
            if (is_array($value)) {
                foreach ($this->renderBranchRecursive($value) as $line) {
                    $output[] = $this->chars[$is_last?self::PREFIX_STEM_LAST:self::PREFIX_STEM] . $line;
                }
            }
        }

        return $output;
    }

    protected function uchars($str)
    {
        return preg_replace_callback("/\\\\u([0-9a-fA-F]{1,4})?/", function ($tok) {
            return html_entity_decode("&#x{$tok[1]};",ENT_COMPAT,"UTF-8");
        }, $str);
    }

}

